function [pf, sz] = pfRead (fname)
%  function pf = pfRead (fname)
%
%  Opens the file and expects it to be of the format outputted by
%  the polytracker vertex editor.  Returns a struct array containing
%  'class' and 'vertices' fields read from the file.
%
% Egon Paztor, Modified to be compatible with new polytracker
%
% atb, Feb 4, 2004
%
% Add new functionalities. Now each object can specify also pose
% parameters.
%
% pf.class
% pf.polygon
% pf.vertices
% pf.view
% pf.labels
%
% labels is another struct with labels that are specific to a particular
% object category. The polygons file can have lines like:
%
% labels {{gender: male} {identity: pedro}}
% 
% and this will produce the next struct:
%
% pf.labels.gender = 'male'
% pf.labels.identity = 'pedro'


polygons = struct('class',{},'polygon',{},'vertices',{});

[fid,msg] = fopen(fname,'r');
if (fid==-1) error(msg); end;

sz = [];
polygonCount = 0;
while(1)
    
    line = getLine(fid);
    switch lower(line(1))
        case '#' % empty line
            fclose(fid);
            pf = polygons;
            return;
        case 's' % size
            [sztoken, line] = getToken(line);
            [psz, line] = getToken(line);
            sz = sscanf(psz, '%d');
        case 'm' % make polygon
            polygonCount = polygonCount + 1;
            [token1,line]   = getToken(line);
            [pPoly,line]    = getToken(line);
            [pClass,line]   = getToken(line);
            [pVert,line]    = getToken(line);
            
            vertices = sscanf(pVert,'%d');
            vertices = reshape(vertices,[2 (length(vertices)/2)]);
            vertices = vertices + 0.5;
            
            polygons(polygonCount).class    = pClass;
            polygons(polygonCount).polygon    = pPoly;
            polygons(polygonCount).vertices = vertices;
        case 'v' % make view
            [token1,line]   = getToken(line);
            [pView, line]   = getToken(line);
            
            pView = sscanf(pView,'%d');
            
            polygons(polygonCount).view    = pView;
        case 'l' % makes labels struct       
            % This allows for producing arbitrary structs of data 
            [token1,line]   = getToken(line);
            [labels,line]   = getToken(line);
            
            Labels = [];
            
            openB = findstr(labels,'{');
            closeB = findstr(labels,'}');
            Nfields = length(openB);
            for nf = 1:Nfields
                getField = labels(openB(nf)+1:closeB(nf)-1);
                L = findstr(getField, ':');
                fieldName = deblank(getField(1:L-1));
                fieldContent = deblank(getField(L+1:end));
                
                Labels = setfield(Labels, fieldName, fieldContent);
            end
            polygons(polygonCount).labels    = Labels;
    end
end;

% -------------------------------------------------------
% read a single word or braced multiword token
% -------------------------------------------------------

function [token, rem] = getToken (s)

pWord.string = s;
pWord.length = length(s);
pWord.loc    = 1;
pWord.eos    = 0;
pWord = getChar(pWord);

token = '';
rem   = '';

while ((pWord.currentChar == ' ') & (pWord.eos == 0))
  pWord = getChar(pWord);
end;

if (pWord.eos == 0) 
  if ((pWord.currentChar == '{') & (pWord.escaped == 0))  
  % - - - - - - - -

    braceDepth = 1;    
    while (pWord.eos == 0)
       pWord = getChar(pWord);  
       if (pWord.currentChar == '{') & (pWord.escaped == 0) 
          braceDepth = braceDepth + 1;
       end;
       if (pWord.currentChar == '}') & (pWord.escaped == 0)  
          braceDepth = braceDepth - 1;
       end;

       if (braceDepth > 0)
          token = [token, pWord.currentChar];
       else
          break;
       end;
    end;
    pWord = getChar(pWord);

  % - - - - - - - -
  else
  % - - - - - - - -
 
    while (((pWord.currentChar ~= ' ') | (pWord.escaped == 1)) & (pWord.eos == 0))
       token = [token, pWord.currentChar];
       pWord = getChar(pWord);
    end;

  end;
  % - - - - - - - -
end;

% --------------------------------------------

if (pWord.eos == 0)
  rem = pWord.string(pWord.loc:end);
end;


% --------------------------------------------
%  Read a character recognizing escaped 
%   braces and escaped spaces.
% --------------------------------------------

function pWord = getChar(pWord)

pWord.currentChar = 0;
pWord.escaped     = 0;

if (pWord.loc <= pWord.length) 

  c = pWord.string(pWord.loc);
  if (c == '\') 
     pWord.escaped = 1;

     pWord.loc = pWord.loc + 1;
     if (pWord.loc <= pWord.length)
        c = pWord.string(pWord.loc);
     else
        c = 0;
     end;
  end;
  pWord.currentChar = c;
  pWord.loc = pWord.loc + 1;

else

  pWord.eos = 1;

end;

% -------------------------------------------------------
% read a line ignoring #-preceded comment lines.
% -------------------------------------------------------

function line = getLine(fid)

line='#';  
while (isstr(line) & (line(1)=='#') & (feof(fid) == 0))
  line = fgetl(fid);
  if (length(line) == 0) line='#'; end;
end;
