function DB = makeDB(imagesFolder, annoFolder, placesFolder)
% Make a database of objects and scenes/places.
% This program has to be run everytime that the annotations are modified.
%
% DB = makeDB(imagesFolder, annoFolder, placesFolder);
%
% The images are assumed to have names like
%   imagesFolder/folder1/foo.jpg
%
% The annotations are assumed to have names like
%   annoFolder/folder1/foo_polygons.txt [may not exist for all files]
%
% The annotation files are created by pfWrite.
%
% Alternatively, the file may be called
%   annoFolder/folder1/foo_bbox.txt [may not exist for all files]
% The bbox.txt files are created by convertUIUCtoBbox.m
%
% Some folders may optionally have a list of place/ scene names
%   placesFolder/folder1.txt [may not exist for all folders]
% See below for the format of the places files.
%
% The output DB is a struct containing the following fields:
% objectNames{i} - list of all possible object names
% objectCounts(i) - number of times this object occurs
% placeNames{i} - list of all possible place names
%% sceneNames{i} - list of all possible scene names
% placeCatNames{i} - list of all possible place category names
% annotated(i) = 0/1
% frames(t).folder  e.g., 'feb19_bldg400_fl6_a'
%          .fileNameImage - e.g., '00001.jpg'
%          .imgSize = [rows cols colors]
%          .placeName
%          .placeCatName 
% If annotated(i) = 1,  the following fields exist (may differ from frame to frame)
%      .object(1).className
%      .object(1).vertices = [xcoords; ycoords]
%      .object(1).bbox = [xmin ymin xmax ymax] = bounding box
%      .object(1).bbox01 as fraction of image size
%      .object(1).area 
%      .object(1).center = [x y]
%      .object(1).view - optional
%      .object(1).labels - optional
%      .object(2).className
%      ...
% e.g. {DB.frame(f).objects.className} prints names of all objects in frame f
%
% SEE ALSO queryDB and showImages
%
% FORMAT OF THE PLACES.TXT FILE:
% There are 3 possible formats:
% 1) for video sequences
% Each row has 3 columns:
%  (1) start frame (2) stop frame (3) place name
% Image files must have names like 0001.jpg, so we can
% figure out the frame number and hence the place by indexing.
%
% 2) for static images all from the same place:
% the places.txt file has a single row of the form
%   1 1000 place_name
% where 1000 is an arbitrary number larger than the number of files
%
% 3) for static images from different places:
% the places.txt file starts with the special row 'filenames'
% and then each row has the form
%   filename place_name
% e.g.,
%  filenames 
%  P1010760.JPG street_foreign
%  P1010761.JPG street_foreign
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Kevin Murphy & Antonio Torralba
%
% MIT-CSAIL Database of Objects and Scenes
% http://web.mit.edu/torralba/www/database.html
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

DB.placesFolder = placesFolder;
DB.imagesFolder = imagesFolder;
DB.annoFolder = annoFolder;

folders = dir(imagesFolder);
folders = {folders.name};
folders = setdiff(folders, {'.', '..'});

fr = 1;

DB.objectNames = {};
DB.frame = [];

for f=1:length(folders)
    folder = folders{f};
    fprintf('makeDB: processing folder %d of %d (%s)\n', f, length(folders), folder);
    names = dir(fullfile(imagesFolder, folder, '*.jpg'));
    names = {names.name};
    
    frameFieldBase.fileNameImage = '';
    frameFieldBase.folder = folder;
    frameFieldBase.placeName = '';
    %frameFieldBase.sceneName = '';
    frameFieldBase.placeCatName = '';
    frameFieldBase.objects = [];
    frameFieldBase.imgSize = [0 0 0];
    frame = repmat(frameFieldBase, [1 length(names)]);
    [placeNames, sceneNames, isVideo] = processPlaceData(imagesFolder, placesFolder, folder);
    
    anotated = 0;
    for fi=1:length(names)
        imgSize = [];
        name = names{fi};
        frame(fi).fileNameImage  = name;

        baseName = strrep(lower(name), '.jpg', '');
        annoName = fullfile(annoFolder, folder, sprintf('%s_polygons.txt', baseName));
        if exist(annoName, 'file')>0
            anotated = anotated+1;
            [obj, objNames, imgSize] = parsePolygonFile(annoName);
            frame(fi).objects = obj;
            DB.annotated(fr) = true;
            DB.objectNames = [DB.objectNames objNames];
        else
            DB.annotated(fr) = false;
        end
        frame(fi).imgSize = imgSize;
        if fi<=length(placeNames)
            frame(fi).placeName = placeNames{fi};
            %frame(fi).sceneName = sceneNames{fi};
            frame(fi).placeCatName = sceneNames{fi};
        end
        fr = fr + 1;
    end
    fprintf('There are %d anotated frames\n', anotated);

    DB.frame = [DB.frame frame];
end

%DB.sceneNames = unique({DB.frame.sceneName});
DB.placeCatNames = unique({DB.frame.placeCatName});
DB.placeNames = unique({DB.frame.placeName});
allObjNames = DB.objectNames;
DB.objectNames = unique(DB.objectNames);
for oi=1:length(DB.objectNames)
    DB.objectCounts(oi) = length(find(strmatch(DB.objectNames{oi}, allObjNames, 'exact')));
end

[foo, k] = sort(DB.objectNames);
objects = DB.objectNames(k);
countsObjects = DB.objectCounts(k);

figure
h = barh(countsObjects);
set(gca, 'YTick', 1:length(objects))
set(gca, 'YTickLabel', objects)
set(gca, 'FontSize', 7)
grid on
title(sprintf('Number of images:%d, Anotated files: %d, number of objects: %d', length(DB.annotated), sum(DB.annotated), length(objects)))
axis tight


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [objects, objNames, sz] = parsePolygonFile(fname)

objects = [];
[pf, sz] = pfRead(fname);
if length(sz)>1
    nrows_img = sz(1);
    ncols_img = sz(2);
end
nObj = size(pf,2);
objNames = {};
for o=1:nObj
    cname = pf(o).class;
    objects(o).className = cname;
    objNames{end+1} = cname;
    v= pf(o).vertices;
    objects(o).vertices = v;
    
    % create bounding box:
    xmin = max([min(v(1,:)) 1]);
    ymin = max([min(v(2,:)) 1]);
    if length(sz)>1
        xmax = min([max(v(1,:)) ncols_img]);
        ymax = min([max(v(2,:)) nrows_img]);
    else
        xmax = max(v(1,:));
        ymax = max(v(2,:));
    end
    bb = [xmin; xmax; ymin; ymax];
    objects(o).bbox = bb;
    if length(sz)>1
        bb(1:2) = bb(1:2) / sz(2);
        bb(3:4) = bb(3:4) / sz(1);
        objects(o).bbox01 = bb;
    end
    if isfield(pf(o), 'view')
        objects(o).view = pf(o).view;
    end 
    if isfield(pf(o), 'labels')
        objects(o).labels = pf(o).labels;
    end 
end

%%%%%%%%%%%%%%%%%%%

function  [placeNames, sceneNames, isVideo] = processPlaceData(imagesFolder, placesFolder, folder);

% placeNames{t} is the name of the place for frame t.
% For a video sequences, many frames may have the same name.

placeFileName = fullfile(placesFolder, sprintf('%s.txt', folder));
if ~exist(placeFileName, 'file')
    placeNames = [];
    sceneNames = [];
    isVideo = 0; % be cautious
    return;
end

names = dir(fullfile(imagesFolder, folder)); 
names = {names.name};
names = setdiff(names, {'.', '..'});

fid = fopen(placeFileName);
tline = fgetl(fid);
fclose(fid);    
staticImages = ~isempty(strfind(tline, 'filenames'));

if staticImages
    [fileNames, placeNames] = textread(placeFileName,  '%s %s');
    fileNames = fileNames(2:end); % skip first line
    placeNames = placeNames(2:end);
    %if ~isequal(names, fileNames)
    %  error('places.txt filenames not equal to image names');
    %end
    [sceneNames, indoors] = placenameToScene(placeNames);
    isVideo = 0;
else
    [initFrame, lastFrame, place_names] = textread(placeFileName,  '%d %d %s');
    Nintervals = size(initFrame,1);
    if (Nintervals == 1) % static
        lastFrame(1) = length(names);
        % in a static folder,  we set the ending frame arbitrarily
        % since we can compute by looking at the iamges
    end
    len = 0;
    Nframes = lastFrame(end);
    placeNames = cell(1, Nframes);
    sceneNames = cell(1, Nframes);
    c = 1;
    for i=1:Nintervals
        if i>1 & (initFrame(i) ~= lastFrame(i-1)+1)
            error(sprintf('folder %s has inconsistency at interval %d\n', folder, i))
        end
        lenInterval = lastFrame(i) - initFrame(i) + 1;
        placeNames(c:c+lenInterval-1) = repmat({place_names{i}}, 1, lenInterval);
        sceneNames(c:c+lenInterval-1) = repmat({placenameToScene(place_names{i})}, 1, lenInterval);
        c = c  + lenInterval;
    end
    isVideo = 1;
end

%%%%%%%%%%%%

function [scenename, indoor] = placenameToScene(placename)
% Convert street_foo_bar to street

if ~iscell(placename)
    placename = {placename}; 
    returnCell = 0;
else
    returnCell = 1;
end

for i=1:length(placename)
    ndx = strfind(placename{i}, '_');
    if isempty(ndx)
        scenename{i} = placename{i};
    else
        scenename{i} = placename{i}(1:ndx-1);
    end
    
    switch scenename{i}
        case {'street', 'plaza'}, 
            indoor(i) = 0;
        otherwise,
            indoor(i) = 1;
    end
end

if ~returnCell
    scenename = scenename{1};
end

