function keys = queryDB(DB, varargin)
% QUERY OF THE IMAGES AND OBJECTS DATABASE
%
% example:
% keys = queryDB(DB, 'findObject', 'screenFrontal')
% keys = queryDB(DB, 'findObject', 'screen*')
%
% This returs all the screens with azimuth = 330
% keys = queryDB(DB, 'findObject', 'screen*', 'findAzimuth', 330)
%
% This returs all the images that contain both screens and keyboards with azimuth = 330
% keys = queryDB(DB, 'findObject', 'screen*',  'findObject', 'keyboard', 'findAzimuth', 330)
%
% OR QUERY:
% keys = queryDB(DB, 'findObject', 'frontalScreen', 'keyboard')
%
% AND QUERY:
% keys = queryDB(DB, 'findObject', 'frontalScreen', 'findObject', 'keyboard')
%
% NEGATIONS:
% this example gives images with frontal screens and no keyboards:
% keys = queryDB(DB, 'findObject', 'frontalScreen', 'findObject', '~keyboard')
% showImages(DB, [keys(:).frame])
%
%LABELS query
% keys = queryDB(DB, 'findObject', 'frontalFace', 'findLabel', 'gender=male')
%  if the symbol "=" is in the string, then 
%
% now, the command findNonObject can be done:
% keys = queryDB(DB, 'findObject', '~frontalScreen')
%
% OTHER QUERIES:
% keys = queryDB(DB, 'findLocation', '400_fl_608')
% keys = queryDB(DB, 'findCategory', 'office')
% keys = queryDB(DB, 'findCategory', '~office')
% keys = queryDB(DB, 'findObject', 'frontalScreen', 'findCategory', 'office')
% keys = queryDB(DB, 'findAnnotated') - checks if objects field is non-empty
% keys = queryDB(DB) returns all keys
% keys = queryDB(DB, 'findfolder', 'static')
% keys = queryDB(DB, 'findfolder', '~static')
% keys = queryDB(DB, ..., 'validFrames', frames) restricts attention to specified frames

query = varargin;
Narguments = length(query);


Nframes = length(DB.frame);
frames = 1:Nframes;

validFrames = frames;
for i=1:Narguments
  if strcmp(query{i}, 'validFrames')
    validFrames = query{i+1};
  end
end

keys.frame=[];
keys.objects=[];
keys = repmat(keys,[1 Nframes]);
for i = 1:Nframes
    keys(i).frame = i;
end

i = 1;
while (i <= Narguments)
    question = varargin{i};
    i = i+1;
    switch lower(question)
    case {'findfolder'}
        target = varargin{i}; i = i+1;
        [target, pos] = parseTarget(target);
        %target = lower(target);
        j = zeros(1,length(frames(:)));
        for t=1:length(frames(:));
            j(t) = length(strfind(DB.frame(frames(t)).folder, target))>0;
        end
        ndx = find(j==pos);        
        %ndx = find(ismember(frames, ndx) == pos); 
        
    case {'findcategory'}
        target = varargin{i}; i = i+1;
        [target, pos] = parseTarget(target);
        ndx =  strmatch(target, {DB.frame.placeCatName});
        ndx = find(ismember(frames, ndx) == pos); 
        
    case {'findlocation'}
        target = varargin{i}; i = i+1;
        [target, pos] = parseTarget(target);
        ndx =  strmatch(target,{DB.frame.placeName});
        ndx = find(ismember(frames, ndx) == pos); 
        
    case {'findannotated'}
        j = zeros(1,length(frames(:))); 
        for t=1:length(frames(:));
            j(t) = isempty(DB.frame(frames(t)).objects);
        end
        ndx = find(j==0);
        
    case {'findobject'}
        j = zeros(1,length(frames(:)));
        object = 1;
        while object == 1 & i <= Narguments
            target = varargin{i};
            i = i+1;
            [target, pos, object, star] = parseTarget(target);
            if object == 0
                i = i-1;
                continue
            else
                for t=1:length(frames(:));
                    entry = DB.frame(frames(t));
                    nObj = length(entry.objects);
                    obj = [];
                    for o=1:nObj
                        if star d=strmatch(target, entry.objects(o).className);
                        else d=strmatch(target, entry.objects(o).className,'exact');
                        end
                        
                        if d
                            obj = [obj o];
                        end
                    end
                    if isempty(obj) == 1-pos
                        keys(frames(t)).objects = obj;
                        j(t) = j(t) + 1;
                    end
                end
                
            end
        end
        ndx = find(j>0);
        
    case {'findazimuth'}
        target = varargin{i};
        [target, pos, object, star] = parseTarget(target);
        if isstr(target)
            azimuth = str2num(target);
        else
            azimuth = target;
        end
        i = i+1;
        j = zeros(1,length(frames(:)));
        for t=1:length(frames(:));
            entry = DB.frame(frames(t));
            nObj = length(entry.objects);
            obj = [];
            for o = keys(frames(t)).objects
                if isfield(entry.objects(o), 'view')
                    pfView = entry.objects(o).view;
                    if length(pfView)>0
                        if pfView(1) == azimuth
                            obj = [obj o];
                        end
                    end
                end
            end
            if isempty(obj) == 1-pos
                keys(frames(t)).objects = obj;
                j(t) = j(t) + 1;
            end
        end
        
        ndx = find(j>0);
        
    case {'findlabel'}
        target = varargin{i};
        [target, pos, object, star] = parseTarget(target);
        L = findstr(target, '=');
        fieldName = deblank(target(1:L-1));
        fieldContent = deblank(target(L+1:end));
        
        i = i+1;
        j = zeros(1,length(frames(:)));
        for t=1:length(frames(:));
            entry = DB.frame(frames(t));
            nObj = length(entry.objects);
            obj = [];
            for o = keys(frames(t)).objects
                % consider only object with the field "labels"
                if isfield(entry.objects(o), 'labels')
                    labels = entry.objects(o).labels;
                    % consider only objects with the field "labels"
                    % non-empty
                    if length(labels)>0
                        % consider only objects the subfield 'fieldName'
                        if isfield(labels, fieldName)
                            field = getfield(labels, fieldName);
                            % and only when the content matches the target
                            if strcmp(field, fieldContent)
                                obj = [obj o];
                            end
                        end
                    end
                end
            end
            if isempty(obj) == 1-pos
                keys(frames(t)).objects = obj;
                j(t) = j(t) + 1;
            end
        end
        
        ndx = find(j>0);
        
    otherwise
        disp(question)
        error('Unknown query command')
    end
    
    frames = frames(ndx);
    frames = intersect(frames, validFrames);
end

keys = keys(frames);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [target, pos, object, star] = parseTarget(target)

pos = 1;
object = 0;
star = 0;
if strcmp(target(1),'~')
    target = target(2:end);
    pos = 0;
end

if length(target)<4 
    object = 1;
elseif  ~strcmp(target(1:4),'find')
    object = 1;
end

if strcmp(target(end),'*')
    target = target(1:end-1);
    star = 1;
end

